using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Text;

using CMS.Forums;
using CMS.GlobalHelper;
using CMS.URLRewritingEngine;
using CMS.UIControls;

public partial class CMSModules_Forums_Controls_PostTree : ForumViewer
{
    #region "Variables"

    /// <summary>
    /// Forum Post tree provider.
    /// </summary>
    protected ForumPostTreeProvider mForumPostTreeProvider = null;

    private bool mAllowReply = true;
    private string mPostPath = "/%";
    private bool mShowApprovePics = false;
    private bool mAdministrationMode = false;
    private int mSelected = 0;
    private bool mSelectOnlyApproved = false;
    private bool mRegularLoad = false;


    /// <summary>
    /// Item CSS class
    /// </summary>
    protected string mItemCssClass = "ThreadPost";

    /// <summary>
    /// If is true unapproved post is higlighted
    /// </summary>
    protected bool mHighlightUnApproved = false;

    /// <summary>
    /// Selcted item CSS class
    /// </summary>
    protected string mSelectedItemCssClass = "ThreadPostSelected";

    #endregion


    #region "Public properties"

    /// <summary>
    /// If is true unapproved post is higlighted
    /// </summary>
    public bool HighlightUnApprove
    {
        get
        {
            return mHighlightUnApproved;
        }
        set
        {
            mHighlightUnApproved = value;
        }
    }


    /// <summary>
    /// Enable subscription
    /// </summary>
    public bool AllowReply
    {
        get
        {
            return mAllowReply;
        }
        set
        {
            mAllowReply = value;
        }
    }


    /// <summary>
    /// Path
    /// </summary>
    public string PostPath
    {
        get
        {
            return mPostPath;
        }
        set
        {
            mPostPath = value;
        }
    }


    /// <summary>
    /// Select only approved
    /// </summary>
    public bool SelectOnlyApproved
    {
        get
        {
            return mSelectOnlyApproved;
        }
        set
        {
            mSelectOnlyApproved = value;
        }
    }


    /// <summary>
    /// Selected path.
    /// </summary>
    public int Selected
    {
        get
        {
            return mSelected;
        }
        set
        {
            mSelected = value;
        }
    }


    /// <summary> AdministrationMode </summary>
    public bool AdministrationMode
    {
        get
        {
            return mAdministrationMode;
        }
        set
        {
            mAdministrationMode = value;
        }
    }


    /// <summary>
    /// Show approved pictures in tree.
    /// </summary>
    public bool ShowApprovePics
    {
        get
        {
            return mShowApprovePics;
        }
        set
        {
            mShowApprovePics = value;
        }
    }


    /// <summary>
    /// Post tree provider that the tree uses
    /// </summary>
    public ForumPostTreeProvider MapProvider
    {
        get
        {
            if (mForumPostTreeProvider == null) { mForumPostTreeProvider = new ForumPostTreeProvider(); }
            return mForumPostTreeProvider;
        }
    }


    /// <summary>
    /// Provides acces to treeElem property
    /// </summary>
    public TreeView TreeView
    {
        get
        {
            return treeElem;
        }
    }


    /// <summary>
    /// Item CSS class
    /// </summary>
    public string ItemCssClass
    {
        get
        {
            return mItemCssClass;
        }
        set
        {
            mItemCssClass = value;
        }
    }


    /// <summary>
    /// Selected item CSS class
    /// </summary>
    public string SelectedItemCssClass
    {
        get
        {
            return mSelectedItemCssClass;
        }
        set
        {
            mSelectedItemCssClass = value;
        }
    }

    #endregion


    #region "Helper methods and properties"

    /// <summary>
    /// Gets or sets the value that indicates whether data has been loaded or current load is 
    /// called on demand
    /// </summary>
    protected bool RegularLoad
    {
        get
        {
            return mRegularLoad;
        }
        set
        {
            mRegularLoad = value;
        }
    }


    /// <summary>
    /// Returns list of required columns if current mode is tree, otherwise returns null => load all columns
    /// </summary>
    /// <returns></returns>
    protected string GetColumns()
    {
        if (ShowMode == ShowModeEnum.TreeMode)
        {
            return "PostSubject, PostIDPath, PostParentID, PostApproved, PostID, PostThreadPostsAbsolute";
        }

        return null;
    }

    #endregion


    #region "Methods"

    /// <summary>
    /// Page load
    /// </summary>
    protected void Page_Load(object sender, EventArgs e)
    {
        treeElem.ImageSet = TreeViewImageSet.Custom;
        treeElem.LineImagesFolder = GetImageUrl("Design/Controls/Tree", IsLiveSite, true);
        if ((this.IsLiveSite && CultureHelper.IsPreferredCultureRTL()) || (!this.IsLiveSite && CultureHelper.IsUICultureRTL()))
        {
            treeElem.LineImagesFolder = GetImageUrl("RTL/Design/Controls/Tree", IsLiveSite, true);
        }
    }


    /// <summary>
    /// OnPreRender
    /// </summary>
    protected override void OnPreRender(EventArgs e)
    {
        ReloadData();

        base.OnPreRender(e);
    }


    /// <summary>
    /// Reload data
    /// </summary>
    public override void ReloadData()
    {
        if (StopProcessing)
        {
            return;
        }

        // Set regular load flag
        this.RegularLoad = true;

        // Set Map provider values
        MapProvider.BindNodeData = true;
        MapProvider.Path = this.PostPath;
        MapProvider.MaxRelativeLevel = this.MaxRelativeLevel;
        MapProvider.SelectOnlyApproved = this.SelectOnlyApproved;
        MapProvider.ForumID = this.ForumID;
        MapProvider.Columns = GetColumns();

        // If tree should be expanded, load all nodes
        if (!this.ExpandTree)
        {
            // None of nodes is selected
            if (Selected <= 0)
            {
                // Combine where conditions if it is necessary
                if (!String.IsNullOrEmpty(this.WhereCondition))
                {
                    MapProvider.WhereCondition = " (" + this.WhereCondition + ") AND PostLevel = 0";
                }
                else
                {
                    MapProvider.WhereCondition = "PostLevel = 0";
                }
            }
            // If some node is selected load all nodes of current most top upper tree
            else
            {
                // Combine where conditions if it is necessary
                if (!String.IsNullOrEmpty(this.WhereCondition))
                {
                    MapProvider.WhereCondition = " (" + this.WhereCondition + ") AND (PostLevel = 0 OR PostIdPath LIKE (SELECT TOP 1 SubString(PostIdPath, 0, " + ForumPostInfoProvider.ForumPostIdLength + 2 + ") FROM Forums_ForumPost WHERE PostID = " + Selected + ") + '/%' )";
                }
                else
                {
                    MapProvider.WhereCondition = "(PostLevel = 0 OR PostIdPath LIKE (SELECT TOP 1 SubString(PostIdPath, 0, " + Convert.ToString(ForumPostInfoProvider.ForumPostIdLength + 2) + ") FROM Forums_ForumPost WHERE PostID = " + Selected + ") + '/%' )";
                }
            }
        }
        else
        {
            MapProvider.WhereCondition = this.WhereCondition;
        }


        // Create root node
        if (((ForumPostTreeNode)MapProvider.RootNode).TreeNode != null)
        {
            this.treeElem.ExpandImageToolTip = ResHelper.GetString("ContentTree.Expand");
            this.treeElem.CollapseImageToolTip = ResHelper.GetString("ContentTree.Collapse");

            this.treeElem.EnableViewState = false;
            this.treeElem.Nodes.Clear();

            // Add root node
            this.treeElem.Nodes.Add(CreateNode((ForumPostTreeNode)MapProvider.RootNode));
            this.treeElem.DataBind();
        }
    }


    /// <summary>
    /// Create node
    /// </summary>
    /// <param name="sourceNode">Node with source data</param>
    protected System.Web.UI.WebControls.TreeNode CreateNode(ForumPostTreeNode sourceNode)
    {
        // Create root node
        System.Web.UI.WebControls.TreeNode newNode = new System.Web.UI.WebControls.TreeNode();

        newNode = sourceNode.TreeNode;
        newNode.NavigateUrl = URLRewriter.CurrentURL + "#";


        // Set node dispalyed tex depend on view mode
        if (AdministrationMode)
        {
            newNode.Text = "<span class=\"" + this.ItemCssClass + "\" onclick=\"ShowPost('-1'); SelectNode('-1',''); \"><span class=\"Name\">" + newNode.Text.Replace("<", "&lt;").Replace(">", "&gt;") + "</span></span>";
        }
        else
        {
            newNode.Text = "<span class=\"" + this.ItemCssClass + "\" onclick=\"ShowPost('-1'); SelectNode('-1',''); \"><span class=\"Name\">" + newNode.Text.Replace("<", "&lt;").Replace(">", "&gt;") + "</span></span>";
        }

        newNode.Select();
        string mPath = String.Empty;

        string originalItemCss = this.ItemCssClass;

        //Get path of current node
        ForumPostTreeNode fptn = MapProvider.GetNodeById(Selected);
        if (fptn != null && fptn.ItemData != null)
        {
            mPath = ((DataRow)fptn.ItemData)["PostIDPath"].ToString();
        }

        // Check whether item ata are defined
        if (sourceNode.ItemData != null)
        {
            // Check whether current item is under selected path
            if (mPath.StartsWith(ValidationHelper.GetString(((DataRow)sourceNode.ItemData)["PostIDPath"], String.Empty)))
            {
                newNode.PopulateOnDemand = true;
                newNode.Expand();
            }
            else
            {
                // Get number of post in current thread
                int numberOfPosts = ValidationHelper.GetInteger(((DataRow)sourceNode.ItemData)["PostThreadPostsAbsolute"], 0);
                // Node has some childs
                if (numberOfPosts > 1)
                {
                    newNode.PopulateOnDemand = true;
                }
                // Node is child node => populate and expand
                else if (numberOfPosts == 0)
                {
                    newNode.PopulateOnDemand = true;
                    newNode.Expand();
                }
            }
        }
        // Root node populate by default
        else
        {
            newNode.PopulateOnDemand = true;
        }

        return newNode;
    }


    /// <summary>
    /// Image delegate
    /// </summary>
    public delegate string GetIconEventHandler(ForumPostTreeNode node);

    /// <summary>
    /// Get image Url
    /// </summary>
    public event GetIconEventHandler OnGetPostIconUrl;


    /// <summary>
    /// On populate create child nodes
    /// </summary>
    protected void treeElem_TreeNodePopulate1(object sender, TreeNodeEventArgs e)
    {
        // Load data on demand only if current populate request is called onDemand
        if (!this.RegularLoad)
        {
            // Set Map provider values
            MapProvider.BindNodeData = true;
            MapProvider.Path = this.PostPath;
            MapProvider.MaxRelativeLevel = this.MaxRelativeLevel;
            MapProvider.SelectOnlyApproved = this.SelectOnlyApproved;
            MapProvider.ForumID = this.ForumID;
            MapProvider.Columns = GetColumns();
            // Set path with dependence on current item
            MapProvider.Path = ForumPostInfoProvider.GetPath(null, ValidationHelper.GetInteger(e.Node.Value, 0)) + "/%";

            // Set regular load flag
            this.RegularLoad = true;

            // Create root node
            if (((ForumPostTreeNode)MapProvider.RootNode).TreeNode != null)
            {
            }
        }

        string mPath = String.Empty;

        string originalItemCss = this.ItemCssClass;

        //Get path of current node
        ForumPostTreeNode fptn = MapProvider.GetNodeById(Selected);
        if (fptn != null && fptn.ItemData != null)
        {
            mPath = ((DataRow)fptn.ItemData)["PostIDPath"].ToString();
        }

        e.Node.ChildNodes.Clear();

        // Get node by nodeId
        int nodeID = ValidationHelper.GetInteger(e.Node.Value, 0);
        ForumPostTreeNode node = MapProvider.GetNodeById(nodeID);

        if (node != null)
        {
            bool expand = false;
            bool isRTL = ((this.IsLiveSite && CultureHelper.IsPreferredCultureRTL()) || (!this.IsLiveSite && CultureHelper.IsUICultureRTL()));

            // Add all child in tree
            foreach (ForumPostTreeNode childNode in node.ChildNodes)
            {
                this.ItemCssClass = originalItemCss;

                if (HighlightUnApprove)
                {
                    if (childNode != null && childNode.ItemData != null && !ValidationHelper.GetBoolean(((DataRow)childNode.ItemData)["PostApproved"], false))
                    {
                        this.ItemCssClass = this.ItemCssClass + " PostUnApproved";
                    }
                }

                System.Web.UI.WebControls.TreeNode newNode = CreateNode(childNode);
                newNode.NavigateUrl = URLRewriter.CurrentURL + "#";

                string imageUrl = "";
                string imageTag = "";

                if (OnGetPostIconUrl != null)
                {
                    imageUrl = OnGetPostIconUrl(childNode);
                }

                if (!String.IsNullOrEmpty(imageUrl))
                {
                    string floating = isRTL ? "float:right;" : "float:left;";
                    imageTag = "<img src=\"" + imageUrl + "\" alt=\"post\" style=\"border:0px;vertical-align:middle; " + floating + " \" />&nbsp;";
                }

                // Create detail of post to string
                StringBuilder sb = new StringBuilder();
                string mId = "";

                if (this.ShowMode != ShowModeEnum.TreeMode)
                {
                    Html32TextWriter writer = new Html32TextWriter(new StringWriter(sb));
                    CMSModules_Forums_Controls_Posts_ForumPost post = (CMSModules_Forums_Controls_Posts_ForumPost)this.Page.LoadControl("~/CMSModules/Forums/Controls/Posts/ForumPost.ascx");
                    post.ID = "forumPost" + newNode.Value;

                    CopyValues(post);

                    post.PostID = ValidationHelper.GetInteger(newNode.Value, 0);

                    post.ReloadData();
                    post.RenderControl(writer);
                    newNode.Text = sb.ToString();
                    mId = this.ClientID + post.PostID.ToString();
                }

                // Post subject
                string mText = newNode.Text;

                // Set display mode
                switch (this.ShowMode)
                {
                    // Dynamic detail mode
                    case ShowModeEnum.DynamicDetailMode:
                        {
                            newNode.Text = "<span style=\"display:block;\" ID=\"" + mId + "\" class=\"" + this.ItemCssClass + "\" onclick=\" SelectNode('" + "Selected" + mId + "','" + mId + "'); return false;\">" + imageTag + HTMLHelper.HTMLEncode(((DataRow)childNode.ItemData)["PostSubject"].ToString()) + "</span>" + "<div ID=\"" + "Selected" + mId + "\" style=\"display: none;\" class=\"TreeSelectedPost\">" + sb.ToString() + "</div>";
                        }
                        break;

                    // Detail mode
                    case ShowModeEnum.DetailMode:
                        newNode.Text = "<div ID=\"" + "Selected" + mId + "\" style=\"display: block;\" class=\"TreeSelectedPost\">" + sb.ToString() + "</div>";
                        break;

                    // Tree mode
                    default:
                        newNode.Text = "<span class=\"" + this.ItemCssClass + "\" onclick=\"ShowPost(" + newNode.Value + " ); SelectNode(this);\">" + imageTag + "<span class=\"Name\">" + HTMLHelper.HTMLEncode(((DataRow)childNode.ItemData)["PostSubject"].ToString()) + "</span></span>";
                        break;
                }

                // Set selected item class
                string mClass = this.SelectedItemCssClass;

                string spanId = "";

                // Higlight selected item in administration mode
                if (AdministrationMode && ValidationHelper.GetInteger(((DataRow)childNode.ItemData)["PostID"], -1) == this.Selected)
                {
                    mClass = this.SelectedItemCssClass;
                    spanId = "id=\"treeSelectedNode\"";
                    //ltlScript.Text = ScriptHelper.GetScript("SelectReloadNode(getElementById("+this.ClientID+"));");
                }

                // highlight selected item
                if ((this.ShowMode == ShowModeEnum.TreeMode) && (ValidationHelper.GetInteger(childNode.TreeNode.Value, -1) == this.Selected))
                {
                    newNode.Text = "<span " + spanId + " class=\"" + mClass + "\" onclick=\"ShowPost(" + newNode.Value + " ); SelectNode(this);\">" + imageTag + "<span class=\"Name\">" + HTMLHelper.HTMLEncode(((DataRow)childNode.ItemData)["PostSubject"].ToString()) + "</span></span>";
                }

                // Colapse this node
                newNode.Collapse();

                // When expand tree is checke expand it
                if (this.ExpandTree)
                {
                    newNode.Expand();
                }

                // Get ForumPostTreeNode of current Childnode 
                ForumPostTreeNode mNode = MapProvider.GetNodeById(Convert.ToInt32(childNode.TreeNode.Value));

                //Expand actual Path
                if (mNode != null && mNode.ItemData != null)
                {
                    string actPath = ((DataRow)mNode.ItemData)["PostIDPath"].ToString();

                    if (mPath.StartsWith(actPath))
                    {
                        expand = true;
                    }
                }

                this.ItemCssClass = originalItemCss;

                //newNode.Collapse();
                e.Node.ChildNodes.Add(newNode);

                if (expand)
                {
                    e.Node.Expand();
                }
            }

            this.ItemCssClass = originalItemCss;
        }
    }


    /// <summary> Render </summary>
    protected override void Render(HtmlTextWriter writer)
    {
        if (!this.StopProcessing)
        {
            base.Render(writer);
        }
    }

    #endregion
}
