using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Data;
using System.Collections;
using System.Threading;
using System.Security.Principal;

using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.Staging;
using CMS.TreeEngine;
using CMS.SiteProvider;
using CMS.WorkflowEngine;
using CMS.PortalEngine;
using CMS.UIControls;
using CMS.DataEngine;

using TreeNode = CMS.TreeEngine.TreeNode;


public partial class CMSModules_Staging_Tools_Tasks_Tasks : CMSStagingPage
{
    #region "Variables"

    protected static Hashtable mLogs = new Hashtable();
    protected static Hashtable mErrors = new Hashtable();
    protected static Hashtable mInfos = new Hashtable();

    protected bool allowView = true;

    private int serverId = 0;
    private int nodeId = 0;
    private int eventId = 0;
    private string eventCode = null;

    protected string viewImage = "";
    protected string deleteImage = "";
    protected string syncImage = "";

    protected string viewTooltip = "";
    protected string deleteTooltip = "";
    protected string syncTooltip = "";

    protected string syncCurrent = null;
    protected string syncSubtree = null;

    string aliasPath = "/";

    int currentSiteId = 0;
    string currentSiteName = null;

    protected CurrentUserInfo currentUser = null;
    protected GeneralConnection mConnection = null;

    #endregion


    #region "Properties"

    /// <summary>
    /// Connection
    /// </summary>
    public GeneralConnection Connection
    {
        get
        {
            if (mConnection == null)
            {
                mConnection = ConnectionHelper.GetConnection();
            }
            return mConnection;
        }
    }


    /// <summary>
    /// Current log context
    /// </summary>
    public LogContext CurrentLog
    {
        get
        {
            return EnsureLog();
        }
    }


    /// <summary>
    /// Current Error
    /// </summary>
    public string CurrentError
    {
        get
        {
            return ValidationHelper.GetString(mErrors["SyncError_" + ctlAsync.ProcessGUID], "");
        }
        set
        {
            mErrors["SyncError_" + ctlAsync.ProcessGUID] = value;
        }
    }


    /// <summary>
    /// Current Info
    /// </summary>
    public string CurrentInfo
    {
        get
        {
            return ValidationHelper.GetString(mInfos["SyncInfo_" + ctlAsync.ProcessGUID], "");
        }
        set
        {
            mInfos["SyncInfo_" + ctlAsync.ProcessGUID] = value;
        }
    }

    #endregion


    #region "Page events"

    protected void Page_Load(object sender, EventArgs e)
    {
        // Get site info
        currentSiteId = CMSContext.CurrentSiteID;
        currentSiteName = CMSContext.CurrentSiteName;

        // Initialize current user for the async actions
        currentUser = CMSContext.CurrentUser;
        
        // Find postback invoker
        string invokerName = Page.Request.Params.Get("__EVENTTARGET");
        Control invokeControl = !string.IsNullOrEmpty(invokerName) ? Page.FindControl(invokerName) : null;
        if (invokeControl == btnSyncComplete)
        {
            SyncComplete();
        }
        else
        {
            if (!IsCallback)
            {
                // check 'Manage servers' permission
                if (!currentUser.IsAuthorizedPerResource("cms.staging", "ManageTasks"))
                {
                    RedirectToAccessDenied("cms.staging", "ManageTasks");
                }

                // Register the dialog script
                ScriptHelper.RegisterClientScriptBlock(this, typeof(string), ScriptHelper.DIALOG_SCRIPT_KEY, ScriptHelper.DialogScript);

                ltlScript.Text += ScriptHelper.GetScript("function ConfirmDeleteTask(taskId) { return confirm(" + ScriptHelper.GetString(ResHelper.GetString("Tasks.ConfirmDelete")) + "); }");
                ltlScript.Text += ScriptHelper.GetScript("function CompleteSync(){" + Page.ClientScript.GetPostBackEventReference(btnSyncComplete, null) + "}");

                serverId = QueryHelper.GetInteger("serverid", 0);
                nodeId = QueryHelper.GetInteger("nodeid", 0);

                aliasPath = "/";

                // Get the document node
                if (nodeId > 0)
                {
                    TreeProvider tree = new TreeProvider(currentUser);
                    TreeNode node = tree.SelectSingleNode(nodeId, TreeProvider.ALL_CULTURES);
                    if (node != null)
                    {
                        aliasPath = node.NodeAliasPath;
                    }
                }
                tasksUniGrid.OnExternalDataBound += tasksUniGrid_OnExternalDataBound;
                tasksUniGrid.OnAction += tasksUniGrid_OnAction;
                tasksUniGrid.OnDataReload += new OnDataReload(tasksUniGrid_OnDataReload);
                tasksUniGrid.PageSize = "15," + tasksUniGrid.PageSize;

                viewImage = GetImageUrl("Design/Controls/UniGrid/Actions/View.png");
                deleteImage = GetImageUrl("Design/Controls/UniGrid/Actions/Delete.png");
                syncImage = GetImageUrl("Design/Controls/UniGrid/Actions/Synchronize.png");

                syncTooltip = ResHelper.GetString("Tasks.SyncTooltip");
                deleteTooltip = ResHelper.GetString("general.delete");
                viewTooltip = ResHelper.GetString("general.view");

                syncCurrent = ResHelper.GetString("Tasks.SyncCurrent");
                syncSubtree = ResHelper.GetString("Tasks.SyncSubtree");

                imgCurrent.ImageUrl = GetImageUrl("CMSModules/CMS_Staging/synccurrent.png");
                imgSubtree.ImageUrl = GetImageUrl("CMSModules/CMS_Staging/syncsubtree.png");

                btnDeleteAll.Text = ResHelper.GetString("Tasks.DeleteAll");
                btnDeleteSelected.Text = ResHelper.GetString("Tasks.DeleteSelected");
                btnSyncAll.Text = ResHelper.GetString("Tasks.SyncAll");
                btnSyncSelected.Text = ResHelper.GetString("Tasks.SyncSelected");

                btnDeleteAll.OnClientClick = "return confirm(" + ScriptHelper.GetString(ResHelper.GetString("Tasks.ConfirmDeleteAll")) + ");";
                btnDeleteSelected.OnClientClick = "return confirm(" + ScriptHelper.GetString(ResHelper.GetString("general.confirmdelete")) + ");";

                titleElem.TitleText = ResHelper.GetString("Synchronization.Title");
                titleElem.TitleImage = GetImageUrl("CMSModules/CMS_Staging/synchronization.png");

                pnlContent.Visible = true;
                btnCancel.Attributes.Add("onclick", "CancelAction(); return false;");
                btnCancel.Text = ResHelper.GetString("General.Cancel");

                pnlLog.Visible = false;
            }

        }

        ctlAsync.OnFinished += ctlAsync_OnFinished;
        ctlAsync.OnError += ctlAsync_OnError;
        ctlAsync.OnRequestLog += ctlAsync_OnRequestLog;
        ctlAsync.OnCancel += ctlAsync_OnCancel;
    }


    protected override void OnPreRender(EventArgs e)
    {
        tasksUniGrid.ReloadData();

        lblError.Visible = (lblError.Text != "");
        lblInfo.Visible = (lblInfo.Text != "");
        base.OnPreRender(e);
    }

    #endregion


    #region "Grid events"
    
    DataSet tasksUniGrid_OnDataReload(string completeWhere, int currentTopN)
    {
        DataSet ds = TaskInfoProvider.SelectDocumentTaskList(currentSiteId, serverId, aliasPath, null, tasksUniGrid.SortDirect);
        if (!DataHelper.DataSourceIsEmpty(ds))
        {
            pnlTasksGrid.Visible = true;
            pnlFooter.Visible = true;
        }
        else
        {
            if (lblInfo.Text == "")
            {
                lblInfo.Text = ResHelper.GetString("Tasks.NoTasks");
            }
            pnlFooter.Visible = false;
            pnlTasksGrid.Visible = false;
        }
        return ds;
    }


    protected object tasksUniGrid_OnExternalDataBound(object sender, string sourceName, object parameter)
    {
        DataRowView dr = null;
        switch (sourceName.ToLower())
        {
            case "tasktitle":
                dr = (DataRowView)parameter;
                return GetDocumentLink(dr["TaskDocumentID"], HTMLHelper.HTMLEncode(TextHelper.LimitLength(dr["TaskTitle"].ToString(), 100)), dr["TaskType"]);

            case "tasktime":
                return DateTime.Parse(parameter.ToString()).ToString();

            case "taskresult":
                dr = (DataRowView)parameter;
                return GetResultLink(dr["FailedCount"], dr["TaskID"]);

            case "taskview":
                ImageButton viewBtn = (ImageButton)sender;
                if (allowView)
                {
                    string taskId = ((DataRowView)((GridViewRow)parameter).DataItem).Row["TaskID"].ToString();
                    string url = ResolveUrl("~/CMSModules/Staging/Tools/Tasks/View.aspx?taskid=") + taskId;
                    viewBtn.OnClientClick = "window.open('" + url + "'); return false;";
                    return viewBtn;
                }
                viewBtn.Visible = false;
                return viewBtn;
        }
        return parameter.ToString();
    }


    protected void tasksUniGrid_OnAction(string actionName, object actionArgument)
    {
        int taskId = ValidationHelper.GetInteger(actionArgument, 0);
        switch (actionName.ToLower())
        {
            case "synchronize":
                try
                {
                    // Run task synchronization
                    string result = StagingHelper.RunSynchronization(taskId, serverId, true, currentSiteId);


                    if (!string.IsNullOrEmpty(result))
                    {
                        lblError.Text = ResHelper.GetString("Tasks.SynchronizationFailed");
                    }
                    else
                    {
                        lblInfo.Text = ResHelper.GetString("Tasks.SynchronizationOK");
                    }
                }
                catch (Exception ex)
                {
                    lblError.Text = ResHelper.GetString("Tasks.SynchronizationFailed") + " " + ex.Message;
                }
                break;

            case "delete":
                // Delete task
                SynchronizationInfoProvider.DeleteSynchronizationInfo(taskId, serverId, currentSiteId);
                break;
        }
    }

    #endregion


    #region "Grid helper methods"

    /// <summary>
    /// Returns the result link for the synchronization log
    /// </summary>
    /// <param name="failedCount">Failed items count</param>
    /// <param name="taskId">Task ID</param>
    protected string GetResultLink(object failedCount, object taskId)
    {
        int count = ValidationHelper.GetInteger(failedCount, 0);
        if (count > 0)
        {
            string logUrl = "log.aspx?taskid=" + taskId + "&serverId=" + serverId;
            return "<a target=\"_blank\" href=\"" + logUrl + "\" onclick=\"modalDialog('" + logUrl + "', 'tasklog', 700, 500); return false;\">" + ResHelper.GetString("Tasks.ResultFailed") + "</a>";
        }
        else
        {
            return "";
        }
    }


    /// <summary>
    /// Returns link for document view
    /// </summary>
    /// <param name="documentId">Document ID</param>
    /// <param name="taskTitle">Task title</param>
    /// <param name="taskType">Type of the task</param>
    protected string GetDocumentLink(object documentId, object taskTitle, object taskType)
    {
        string title = ValidationHelper.GetString(taskTitle, "");
        string type = ValidationHelper.GetString(taskType, "").ToLower();
        int docId = ValidationHelper.GetInteger(documentId, 0);

        if ((type != "deletedoc") && (type != "deleteallculutres"))
        {
            string viewMode = ViewModeCode.LiveSite.ToString();

            // For publish tasks display document in preview mode
            if ((type == "publishdoc") || (type == "archivedoc"))
            {
                viewMode = ViewModeCode.Preview.ToString();
            }

            // Get document url
            string docUrl = ResolveUrl(CMSContext.GetDocumentUrl(docId)) + "?viewmode=" + viewMode;
            return "<a target=\"_blank\" href=\"" + docUrl + "\">" + HTMLHelper.HTMLEncode(title) + "</a>";
        }
        return title;
    }

    #endregion


    #region "Button handling"

    protected void btnSyncAll_Click(object sender, EventArgs e)
    {
        RunAsync(SynchronizeAll);
    }


    private void SyncComplete()
    {
        RunAsync(SynchronizeComplete);
    }


    protected void btnSyncSelected_Click(object sender, EventArgs e)
    {
        ArrayList list = tasksUniGrid.SelectedItems;
        if (list.Count > 0)
        {
            ctlAsync.Parameter = list;
            RunAsync(SynchronizeSelected);
        }
    }


    protected void btnDeleteAll_Click(object sender, EventArgs e)
    {
        // Process all records
        DataSet ds = TaskInfoProvider.SelectDocumentTaskList(currentSiteId, serverId, aliasPath, null, null);
        if (!DataHelper.DataSourceIsEmpty(ds))
        {
            foreach (DataRow row in ds.Tables[0].Rows)
            {
                // Get ID
                int taskId = ValidationHelper.GetInteger(row["TaskID"], 0);
                if (taskId > 0)
                {
                    SynchronizationInfoProvider.DeleteSynchronizationInfo(taskId, serverId, currentSiteId);
                }
            }
        }

        lblInfo.Text = ResHelper.GetString("Tasks.DeleteOK");
        tasksUniGrid.ResetSelection();
    }


    protected void btnDeleteSelected_Click(object sender, EventArgs e)
    {
        ArrayList list = tasksUniGrid.SelectedItems;
        foreach (string task in list)
        {
            int taskId = ValidationHelper.GetInteger(task, 0);
            if (taskId > 0)
            {
                SynchronizationInfoProvider.DeleteSynchronizationInfo(taskId, serverId, currentSiteId);
            }
        }
        lblInfo.Text = ResHelper.GetString("Tasks.DeleteOK");
        tasksUniGrid.ResetSelection();
    }


    protected void btnCurrent_Click(object sender, EventArgs e)
    {
        RunAsync(SynchronizeCurrent);
    }


    protected void btnSubtree_Click(object sender, EventArgs e)
    {
        RunAsync(SynchronizeSubtree);
    }

    #endregion


    #region "Async methods"

    public void SynchronizeComplete(object parameter)
    {
        try
        {
            eventCode = "SYNCCOMPLETE";
            string result = null;

            int sid = serverId;
            if (sid <= 0)
            {
                sid = SynchronizationInfoProvider.ENABLED_SERVERS;
            }

            AddLog(ResHelper.GetString("Synchronization.LoggingTasks"));

            // Synchronize root node
            ArrayList tasks = DocumentHelper.LogSynchronization(CMSContext.CurrentSiteName, "/", TaskTypeEnum.UpdateDocument, null, sid, false, false);

            AddLog(ResHelper.GetString("Synchronization.RunningTasks"));

            // Run the synchronization
            foreach (TaskInfo task in tasks)
            {
                AddLog(string.Format(ResHelper.GetAPIString("synchronization.running", "Processing '{0}' task"), HTMLHelper.HTMLEncode(task.TaskTitle)));
                result += StagingHelper.RunSynchronization(task.TaskID, serverId, true, currentSiteId);
            }

            AddLog(ResHelper.GetString("Synchronization.LoggingTasks"));

            // Synchronize subnodes
            tasks = DocumentHelper.LogSynchronization(CMSContext.CurrentSiteName, "/%", TaskTypeEnum.UpdateDocument, null, sid, false, false);

            AddLog(ResHelper.GetString("Synchronization.RunningTasks"));

            // Run the synchronization
            foreach (TaskInfo task in tasks)
            {
                AddLog(string.Format(ResHelper.GetAPIString("synchronization.running", "Processing '{0}' task"), HTMLHelper.HTMLEncode(task.TaskTitle)));
                result += StagingHelper.RunSynchronization(task.TaskID, serverId, true, currentSiteId);
            }


            if (!string.IsNullOrEmpty(result))
            {
                CurrentError = ResHelper.GetString("Tasks.SynchronizationFailed");
                AddErrorLog(CurrentError, result);
            }
            else
            {
                CurrentInfo = ResHelper.GetString("Tasks.SynchronizationOK");
                AddLog(CurrentInfo);
            }
        }
        catch (ThreadAbortException)
        {
            CurrentInfo = ResHelper.GetString("Tasks.SynchronizationCanceled");
            AddLog(CurrentInfo);
        }
        catch (Exception ex)
        {
            CurrentError = ResHelper.GetString("Tasks.SynchronizationFailed") + ": " + ex.Message;
            AddErrorLog(CurrentError);
        }
    }


    public void SynchronizeAll(object parameter)
    {
        try
        {
            string result = string.Empty;
            eventCode = "SYNCALLDOCS";

            AddLog(ResHelper.GetString("Synchronization.RunningTasks"));

            // Process all records
            DataSet ds = TaskInfoProvider.SelectDocumentTaskList(currentSiteId, serverId, aliasPath, null, null);
            if (!DataHelper.DataSourceIsEmpty(ds))
            {
                foreach (DataRow row in ds.Tables[0].Rows)
                {
                    string taskTitle = ValidationHelper.GetString(row["TaskTitle"], "");
                    AddLog(string.Format(ResHelper.GetAPIString("synchronization.running", "Processing '{0}' task"), HTMLHelper.HTMLEncode(taskTitle)));

                    // Get ID
                    int taskId = ValidationHelper.GetInteger(row["TaskID"], 0);
                    if (taskId > 0)
                    {
                        result += StagingHelper.RunSynchronization(taskId, serverId, true, currentSiteId);
                    }
                }
            }

            if (result != string.Empty)
            {
                CurrentError = ResHelper.GetString("Tasks.SynchronizationFailed");
                AddErrorLog(CurrentError, result);
            }
            else
            {
                CurrentInfo = ResHelper.GetString("Tasks.SynchronizationOK");
                AddLog(CurrentInfo);
            }
        }
        catch (ThreadAbortException)
        {
            CurrentInfo = ResHelper.GetString("Tasks.SynchronizationCanceled");
            AddLog(CurrentInfo);
        }
        catch (Exception ex)
        {
            CurrentError = ResHelper.GetString("Tasks.SynchronizationFailed") + ": " + ex.Message;
            AddErrorLog(CurrentError);
        }
    }


    protected void SynchronizeSubtree(object parameter)
    {
        try
        {
            // Synchronize current node first
            SynchronizeCurrent(false);
            string result = CurrentError;

            eventCode = "SYNCSUBTREE";
            if (result == "")
            {
                // Synchronize subnodes
                int sid = serverId;
                if (sid <= 0)
                {
                    sid = SynchronizationInfoProvider.ENABLED_SERVERS;
                }

                AddLog(ResHelper.GetString("Synchronization.LoggingTasks"));

                // Get the tasks
                ArrayList tasks = DocumentHelper.LogSynchronization(currentSiteName, aliasPath.TrimEnd('/') + "/%", TaskTypeEnum.UpdateDocument, null, sid, false, false);

                AddLog(ResHelper.GetString("Synchronization.RunningTasks"));

                // Run the synchronization
                foreach (TaskInfo task in tasks)
                {
                    AddLog(string.Format(ResHelper.GetAPIString("synchronization.running", "Processing '{0}' task"), HTMLHelper.HTMLEncode(task.TaskTitle)));
                    result += StagingHelper.RunSynchronization(task.TaskID, serverId, true, currentSiteId);
                }
            }

            if (result != "")
            {
                CurrentError = ResHelper.GetString("Tasks.SynchronizationFailed");
                AddErrorLog(CurrentError, result);
            }
            else
            {
                CurrentInfo = ResHelper.GetString("Tasks.SynchronizationOK");
                AddLog(CurrentInfo);
            }
        }
        catch (ThreadAbortException)
        {
            CurrentInfo = ResHelper.GetString("Tasks.SynchronizationCanceled");
            AddLog(CurrentInfo);
        }
        catch (Exception ex)
        {
            CurrentError = ResHelper.GetString("Tasks.SynchronizationFailed") + ": " + ex.Message;
            AddErrorLog(CurrentError);
        }
    }


    public void SynchronizeSelected(object parameter)
    {
        if (parameter == null)
        {
            return;
        }

        eventCode = "SYNCSELECTEDDOCS";
        ArrayList list = (ArrayList)parameter;
        try
        {
            AddLog(ResHelper.GetString("Synchronization.RunningTasks"));

            string result = "";
            foreach (object taskIdObj in list)
            {
                int taskId = ValidationHelper.GetInteger(taskIdObj, 0);
                if (taskId > 0)
                {
                    TaskInfo task = TaskInfoProvider.GetTaskInfo(taskId);
                    if (task != null)
                    {
                        AddLog(string.Format(ResHelper.GetAPIString("synchronization.running", "Processing '{0}' task"), HTMLHelper.HTMLEncode(task.TaskTitle)));
                        result += StagingHelper.RunSynchronization(taskId, serverId, true, currentSiteId);
                    }
                }
            }

            if (result != "")
            {
                CurrentError = ResHelper.GetString("Tasks.SynchronizationFailed");
                AddErrorLog(CurrentError, result);
            }
            else
            {
                CurrentInfo = ResHelper.GetString("Tasks.SynchronizationOK");
                AddLog(CurrentInfo);
            }
        }
        catch (ThreadAbortException)
        {
            CurrentInfo = ResHelper.GetString("Tasks.SynchronizationCanceled");
            AddLog(CurrentInfo);
        }
        catch (Exception ex)
        {
            CurrentError = ResHelper.GetString("Tasks.SynchronizationFailed") + ": " + ex.Message;
            AddErrorLog(CurrentError);
        }
    }


    /// <summary>
    /// Synchronizes the current document
    /// </summary>
    private void SynchronizeCurrent(object parameter)
    {
        eventCode = "SYNCCURRENTDOC";
        bool finish = ValidationHelper.GetBoolean(parameter, true);

        int sid = serverId;
        if (sid <= 0)
        {
            sid = SynchronizationInfoProvider.ENABLED_SERVERS;
        }

        AddLog(ResHelper.GetString("Synchronization.LoggingTasks"));

        // Get the tasks
        ArrayList tasks = DocumentHelper.LogSynchronization(currentSiteName, aliasPath, TaskTypeEnum.UpdateDocument, null, sid, false, false);

        AddLog(ResHelper.GetString("Synchronization.RunningTasks"));

        string result = "";
        // Run the synchronization
        foreach (TaskInfo task in tasks)
        {
            AddLog(string.Format(ResHelper.GetAPIString("synchronization.running", "Processing '{0}' task"), HTMLHelper.HTMLEncode(task.TaskTitle)));
            result += StagingHelper.RunSynchronization(task.TaskID, serverId, true, currentSiteId);
        }

        if (finish)
        {
            if (result != "")
            {
                CurrentError = ResHelper.GetString("Tasks.SynchronizationFailed");
                AddErrorLog(CurrentError, result);
            }
            else
            {
                CurrentInfo = ResHelper.GetString("Tasks.SynchronizationOK");
                AddLog(CurrentInfo);
            }
        }
    }

    #endregion


    #region "Async processing"

    protected void ctlAsync_OnRequestLog(object sender, EventArgs e)
    {
        ctlAsync.Log = CurrentLog.Log;
    }


    protected void ctlAsync_OnError(object sender, EventArgs e)
    {
        tasksUniGrid.ResetSelection();

        lblError.Text = CurrentError;
        lblInfo.Text = CurrentInfo;
    }


    protected void ctlAsync_OnFinished(object sender, EventArgs e)
    {
        tasksUniGrid.ResetSelection();

        lblError.Text = CurrentError;
        lblInfo.Text = CurrentInfo;
    }


    protected void ctlAsync_OnCancel(object sender, EventArgs e)
    {
        tasksUniGrid.ResetSelection();

        ltlScript.Text += ScriptHelper.GetScript("var __pendingCallbacks = new Array();");

        lblError.Text = CurrentError;
        lblInfo.Text = CurrentInfo;
    }


    protected void RunAsync(AsyncAction action)
    {
        pnlLog.Visible = true;
        CurrentLog.Dispose();
        EnsureLog();
        CurrentError = "";
        CurrentInfo = "";
        eventId = 0;
        pnlContent.Visible = false;

        ctlAsync.RunAsync(action, WindowsIdentity.GetCurrent());
    }


    /// <summary>
    /// Adds the log information
    /// </summary>
    /// <param name="newLog">New log information</param>
    protected void AddLog(string newLog)
    {
        EnsureLog();
        LogContext.AppendLine(newLog);
        AddEventLog(CurrentLog.Log, "I");
    }


    /// <summary>
    /// Ensures the logging context
    /// </summary>
    protected LogContext EnsureLog()
    {
        LogContext log = LogContext.EnsureLog(ctlAsync.ProcessGUID);
        log.Reversed = true;
        log.LineSeparator = "<br />";
        return log;
    }


    /// <summary>
    /// Adds the log error
    /// </summary>
    /// <param name="newLog">New log information</param>
    protected void AddErrorLog(string newLog)
    {
        AddErrorLog(newLog, null);
    }


    /// <summary>
    /// Adds the log error
    /// </summary>
    /// <param name="newLog">New log information</param>
    /// <param name="errorMessage">Error message</param>
    protected void AddErrorLog(string newLog, string errorMessage)
    {
        LogContext.AppendLine(newLog);

        string logMessage = string.Empty;

        if (errorMessage != null)
        {
            logMessage = errorMessage + "<br />" + CurrentLog.Log;
        }
        AddEventLog(logMessage, "E");
    }


    protected void AddEventLog(string logMessage, string eventType)
    {
        if (eventId == 0)
        {
            try
            {
                // New event declaration
                IDataClass newEvent = DataClassFactory.NewDataClass("CMS.EventLog");

                newEvent.SetValue("EventType", eventType);
                newEvent.SetValue("EventTime", DateTime.Now);
                newEvent.SetValue("Source", "Staging");
                newEvent.SetValue("EventCode", eventCode);
                newEvent.SetValue("UserID", currentUser.UserID);
                newEvent.SetValue("UserName", currentUser.UserName);
                newEvent.SetValue("EventDescription", logMessage);

                // Insert a new event
                newEvent.Insert();

                eventId = Convert.ToInt32(newEvent.GetValue("EventID"));
            }
            catch
            {
                // Unable to log into eventlog
            }
        }
        else
        {
            // Get EventLog
            IDataClass newEvent = DataClassFactory.NewDataClass("CMS.EventLog", eventId, Connection);
            if (newEvent != null)
            {
                newEvent.SetValue("EventDescription", logMessage);
                newEvent.SetValue("EventCode", eventCode);
                newEvent.SetValue("SiteID", currentSiteId);
                newEvent.SetValue("EventType", eventType);

                // Update event
                newEvent.Update();
            }
        }
    }

    #endregion
}